#ifndef INCLUDE__InformationManager_hxx
#define INCLUDE__InformationManager_hxx

#include <boost/signals2.hpp>
#include <functional>
#include <string>

class Disassembler;
class Function;

class InformationManager {
public:
	boost::signals2::connection
	connect_new_function_signal(std::function<void(Function*)> f) {
		return new_function_signal.connect(f);
	}

	void signal_new_function(Function* f) {
		new_function_signal(f);
	}

	boost::signals2::connection
	connect_new_dyn_symbol_signal(std::function<void(const std::string& name)> f) {
		return new_dyn_symbol_signal.connect(f);
	}

	void signal_new_dyn_symbol(const std::string& f) {
		new_dyn_symbol_signal(f);
	}

	boost::signals2::connection
	connect_reset_signal(std::function<void ()> f) {
		return reset_signal.connect(f);
	}

	// boost::signals2::connection
	// connect_information_added_signal(uint64_t begin, uint64_t end,
	//                                  std::function<void (uint64_t)>) {

	// }

	Disassembler* getDisassembler() {
		return disassembler.get();
	}

	void reset(const std::string& filename);
private:
	boost::signals2::signal<void ()> reset_signal;
	boost::signals2::signal<void (Function*)> new_function_signal;
	boost::signals2::signal<void (const std::string& name)> new_dyn_symbol_signal;
	std::unique_ptr<Disassembler> disassembler;
};

#endif /* INCLUDE__InformationManager_hxx */
